package cz.jzitnik.game.entities.items;

import cz.jzitnik.game.annotations.BlockRegistry;
import cz.jzitnik.game.annotations.EntityRegistry;
import cz.jzitnik.game.annotations.ItemRegistry;
import cz.jzitnik.game.entities.Block;
import org.reflections.Reflections;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Optional;
import java.util.Set;
import java.util.List;

public class ItemBlockSupplier {
    private static final HashMap<String, Constructor<Block>> registeredBlocks = new HashMap<>();
    private static final HashMap<String, Constructor<Item>> registeredItems = new HashMap<>();
    private static final HashMap<String, Constructor<Block>> registeredEntities = new HashMap<>();
    private static final HashMap<String, String> dropsList = new HashMap<>();

    static {
        registerItems();
        registerBlocks();
        registerEntities();
    }

    private static void registerBlocks() {
        Reflections reflections = new Reflections("cz.jzitnik.game.entities.items.registry");
        Set<Class<?>> blockClasses = reflections.getTypesAnnotatedWith(BlockRegistry.class);

        for (Class<?> clazz : blockClasses) {
            try {
                Constructor<Block> blockInstance = (Constructor<Block>) clazz.getDeclaredConstructor();
                BlockRegistry annotation = clazz.getAnnotation(BlockRegistry.class);
                registeredBlocks.put(annotation.value(), blockInstance);

                String dropKey = annotation.drops().isEmpty() ? annotation.value() : annotation.drops();
                dropsList.put(annotation.value(), dropKey);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private static void registerEntities() {
        Reflections reflections = new Reflections("cz.jzitnik.game.entities.items.registry");
        Set<Class<?>> entityClasses = reflections.getTypesAnnotatedWith(EntityRegistry.class);

        for (Class<?> clazz : entityClasses) {
            try {
                Constructor<Block> entityInstance = (Constructor<Block>) clazz.getDeclaredConstructor();
                EntityRegistry annotation = clazz.getAnnotation(EntityRegistry.class);
                registeredEntities.put(annotation.value(), entityInstance);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private static void registerItems() {
        Reflections reflections = new Reflections("cz.jzitnik.game.entities.items.registry");
        Set<Class<?>> itemClasses = reflections.getTypesAnnotatedWith(ItemRegistry.class);

        for (Class<?> clazz : itemClasses) {
            try {
                Constructor<Item> itemInstance = (Constructor<Item>) clazz.getDeclaredConstructor();
                ItemRegistry annotation = clazz.getAnnotation(ItemRegistry.class);
                registeredItems.put(annotation.value(), itemInstance);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public static Block getBlock(String key) {
        try {
            Block block = registeredBlocks.get(key).newInstance();
            String dropKey = dropsList.get(key);

            if (dropKey != null && registeredItems.containsKey(dropKey)) {
                Item dropItem = getItem(dropKey);
                if (dropKey.equals(block.getBlockId())) {
                    dropItem.setBlock(Optional.of(block));
                } else {
                    dropItem.setBlock(Optional.of(getBlock(dropKey)));
                }
                block.setDrops(List.of(dropItem));
            }

            return block;
        } catch (InstantiationException | IllegalAccessException | InvocationTargetException e) {
            throw new RuntimeException(e);
        }
    }

    public static Item getItem(String key) {
        try {
            Item item = registeredItems.get(key).newInstance();
            if (registeredBlocks.containsKey(key)) {
                Block block = registeredBlocks.get(key).newInstance();
                item.setBlock(Optional.of(block));
                block.setDrops(List.of(item));
            }
            return item;
        } catch (InstantiationException | InvocationTargetException | IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }

    public static Block getEntity(String key) {
        try {
            return registeredEntities.get(key).newInstance();
        } catch (InstantiationException | IllegalAccessException | InvocationTargetException e) {
            throw new RuntimeException(e);
        }
    }
}
